@@ -230,9 +230,11 @@ class GoogleWifi:
       systems[this_system["id"]] = this_system
 
       system_status = await self.get_status(this_system["id"])
+      system_metrics = await self.get_realtime_metrics(this_system["id"])
 
       try:
         systems[this_system["id"]]["status"] = system_status["wanConnectionStatus"]
+        systems[this_system["id"]]["groupTraffic"] = system_metrics.get("groupTraffic",None)
       except KeyError as error:
         raise GoogleWifiException(error)
 
@@ -261,10 +263,12 @@ class GoogleWifi:
       devices_list = await self.get_devices(this_system["id"])
       
       devices = {}
+      station_ids = []
       
       try:
         for this_device in devices_list["stations"]:
           devices[this_device["id"]] = this_device
+          station_ids.append(this_device["id"])
           device_paused = False
 
           if blocking_policies.get(this_device["id"]):
@@ -277,6 +281,15 @@ class GoogleWifi:
       except KeyError as error:
         raise GoogleWifiException(error)
 
+      sensitive_info = await self.get_sensitive_info(system_id=this_system["id"], station_ids=station_ids)                                        
+      for this_station in sensitive_info:
+        if this_station["stationId"] in devices:
+          devices[this_station["stationId"]]["macAddress"] = this_station["macAddress"]
+
+      for this_station in system_metrics["stationMetrics"]:
+        if this_station["station"]["id"] in devices:
+          devices[this_station["station"]["id"]]["traffic"] = this_station["traffic"]
+
       systems[this_system["id"]]["devices"] = devices
 
       return systems
@@ -477,7 +490,7 @@ class GoogleWifi:
 
     if await self.connect():
       url = f"https://{host}:8443/setup/bluetooth/scan"
-      data = {"enable": True, "clear_results": False, "timeout": 5}
+      data = {"enable": True, "clear_results": True, "timeout": 5}
       headers = GH_HEADERS
       headers["Host"] = host
       headers["cast-local-authorization-token"] = token
@@ -489,10 +502,142 @@ class GoogleWifi:
       
       response = await self.get_api(url=url,headers=headers)
 
-      if response:
-        return response
-      else:
-        raise GoogleHomeUpdateFailed()
+      return response
+
+  async def create_wan_speedtest(self, system_id:str):
+    """Start a speed test operation on a system."""
+    if await self.connect():
+      url = f"https://googlehomefoyer-pa.googleapis.com/v2/groups/{system_id}/wanSpeedTest"
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      payload = {}
+      params = (
+        ('prettyPrint', 'false'),
+      )
+
+      response = await self.post_api(url=url,headers=headers,payload=payload,params=params)
+      operation_id = response["operation"]["operationId"]
+
+      return operation_id
+
+  async def check_operation(self, operation_id: str):
+    """Check the status of a speed test operation."""
+    if await self.connect():
+      url = f"https://googlehomefoyer-pa.googleapis.com/v2/operations/{operation_id}"
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      payload = {}
+      params = (
+        ('prettyPrint', 'false'),
+      )
+
+      return await self.get_api(url=url,headers=headers,payload=payload,params=params)
+
+  async def speed_test_results(self, system_id:str):
+    """Retrieve the speed test results."""
+    if await self.connect():
+      url = f"https://googlehomefoyer-pa.googleapis.com/v2/groups/{system_id}/speedTestResults"
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      payload = {}
+      params = (
+        ('prettyPrint', 'false'),
+        ('maxResultCount', 1)
+      )
+
+      response = await self.get_api(url=url,headers=headers,payload=payload,params=params)
+
+      return response["speedTestResults"]
+
+  async def run_speed_test(self, system_id:str):
+    """Run a speed test and return the results."""
+    if await self.connect():
+      operation_id = await self.create_wan_speedtest(system_id=system_id)
+
+      status = await self.check_operation(operation_id)
+      status = status["operationState"]
+
+      while status != "DONE":
+        status = await self.check_operation(operation_id)
+        status = status["operationState"]
+        await asyncio.sleep(5)
+
+      results = await self.speed_test_results(system_id=system_id)
+      return results[0]
+      
+  async def start_retrieve_sensitive_info(self, system_id:str, station_ids:list):
+    """Start the request to return the device sensitive information."""
+    if await self.connect():
+      url = f"https://googlehomefoyer-pa.googleapis.com/v2/groups/{system_id}/stations/operations/sensitiveInfo"
+
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      json_payload = {"stationIds":station_ids}
+      params = (
+        ('prettyPrint', 'false'),
+      )
+
+      response = await self.post_api(url=url,headers=headers,json_payload=json_payload,params=params)
+      operation_id = response["operation"]["operationId"]
+
+      return operation_id
+
+  async def sensitive_info_results(self, operation_id:str):
+    """Return the results of the sensitive info request."""
+    if await self.connect():
+      url = f"https://googlehomefoyer-pa.googleapis.com/v2/operations/{operation_id}/sensitiveInfo"
+      params = (
+        ('prettyPrint', 'false'),
+      )
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      payload = {}
+
+      return await self.get_api(url=url, headers=headers, payload=payload, params=params)
+
+  async def get_sensitive_info(self, system_id:str, station_ids:list):
+    """Return a full set of sensitive info on the system."""
+    if await self.connect():
+      operation_id = await self.start_retrieve_sensitive_info(
+        system_id=system_id,station_ids=station_ids
+      )
+
+      status = await self.check_operation(operation_id)
+      status = status["operationState"]
+
+      while status != "DONE":
+        status = await self.check_operation(operation_id)
+        status = status["operationState"]
+        await asyncio.sleep(5)
+
+      results = await self.sensitive_info_results(operation_id=operation_id)
+      return results["stationSensitiveInfos"]
+
+  async def get_realtime_metrics(self, system_id:str):
+    """Return real-time metrics from the system."""
+    if await self.connect():
+      url =  f"https://googlehomefoyer-pa.googleapis.com/v2/groups/{system_id}/realtimeMetrics"
+      params = (
+        ('prettyPrint', 'false'),
+      )
+      headers = {
+        "Content-Type": "application/json; charset=utf-8",
+        "Authorization": f"Bearer {self._api_token}"
+      }
+      payload = {}
+
+      return await self.get_api(url=url, headers=headers, payload=payload, params=params)
+    
 
 class GoogleWifiException(Exception):
   """Platform not ready exception."""
